#!/usr/bin/perl

# addosisrefs.pl, Version 1.0
# Licensed under the standard BSD license:
# Copyright (C) 2008 Daniel Owens
#
# This code is based in part on gbf2osis.pl 
# Copyright (c) 2005-2007 CrossWire Bible Society <http://www.crosswire.org/>
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
#     * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of the CrossWire Bible Society nor the names of
#       its contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

## For general inquiries, comments, suggestions, bug reports, etc. email:
## sword-support@crosswire.org

#########################################################################

# use strict;
# use warnings;
# use diagnostics;

# Give instructions for use of the script and command line syntax
if (scalar(@ARGV) < 1) {
	print "\nThis Perl script adds some line breaks to the input OSIS XML \nfile to make it more readable and formats osisRefs according \nto the OSIS 2.1.1 schema after an OpenOffice Writer document \nis exported using the oowriter2osis export filter.\n\nInput reference formats that this script handles include the \nfollowing: \n\n    Genesis 1\n    Genesis 1:1\n    Genesis 1-2\n    Genesis 1:1-2:1\n\nReferences spanning multiple books need to be manually corrected. \nAlso, Bible book names and abbreviations that are converted to \nOSIS abbreviations include only a select set of English names \nand abbreviations.\n\nSyntax: addosisrefs.pl <input filename>\nOutput filename is new_<input filename>\n\n";
	exit (-1);
}

# Specifies outputFilename
my $outputFilename = "new_$ARGV[0]"; 

# Create output file
open (OUTF,">$outputFilename") or die "Could not open file $ARGV[1] for writing.";

# Get input text
open (INF, $ARGV[0]);
my @data = <INF>;
close (INF);

foreach $line (@data) {
# Adds line breaks to make the xml file more readable
	$line =~ s/<osis/\n<osis/g;
	$line =~ s/<\/header>/<\/header>\n/g;
	$line =~ s/<refSystem>Bible<\/refSystem><\/work>/\n<refSystem>Bible<\/refSystem>\n<\/work>/g;
	$line =~ s/<div/\n<div/g;
	$line =~ s/<\/div>/\n<\/div>/g;
	$line =~ s/"><title/">\n<title/g;
	$line =~ s/<\/osisT/\n<\/osisT/g;
# Replaces Bible book names with OSIS abbreviations.
	$line =~ s/osisRef="Genesis /osisRef="Gen./g;
	$line =~ s/osisRef="Ge /osisRef="Gen./g;
	$line =~ s/osisRef="Gn /osisRef="Gen./g;
	$line =~ s/osisRef="Exodus /osisRef="Exod./g;
	$line =~ s/osisRef="Ex /osisRef="Exod./g;
	$line =~ s/osisRef="Leviticus /osisRef="Lev./g;
	$line =~ s/osisRef="Lev /osisRef="Lev./g;
	$line =~ s/osisRef="Lv /osisRef="Lev./g;
	$line =~ s/osisRef="Numbers /osisRef="Num./g;
	$line =~ s/osisRef="Nu /osisRef="Num./g;
	$line =~ s/osisRef="Deuteronomy /osisRef="Deut./g;
	$line =~ s/osisRef="De /osisRef="Deut./g;
	$line =~ s/osisRef="Dt /osisRef="Deut./g;
	$line =~ s/osisRef="Joshua /osisRef="Josh./g;
	$line =~ s/osisRef="Jos /osisRef="Josh./g;
	$line =~ s/osisRef="Judges /osisRef="Judg./g;
	$line =~ s/osisRef="Judg /osisRef="Judg./g;
	$line =~ s/osisRef="Jdg /osisRef="Judg./g;
	$line =~ s/osisRef="Ruth /osisRef="Ruth./g;
	$line =~ s/osisRef="Ru /osisRef="Ruth./g;
	$line =~ s/osisRef="1 Samuel /osisRef="1Sam./g;
	$line =~ s/osisRef="1 Sa /osisRef="1Sam./g;
	$line =~ s/osisRef="1Sa /osisRef="1Sam./g;
	$line =~ s/osisRef="2 Samuel /osisRef="2Sam./g;
	$line =~ s/osisRef="2 Sa /osisRef="2Sam./g;
	$line =~ s/osisRef="2Sa /osisRef="2Sam./g;
	$line =~ s/osisRef="1 Kings /osisRef="1Kgs./g;
	$line =~ s/osisRef="1 Ki /osisRef="1Kgs./g;
	$line =~ s/osisRef="1Ki /osisRef="1Kgs./g;
	$line =~ s/osisRef="2 Kings /osisRef="2Kgs./g;
	$line =~ s/osisRef="2 Ki /osisRef="2Kgs./g;
	$line =~ s/osisRef="2Ki /osisRef="2Kgs./g;
	$line =~ s/osisRef="1 Chronicles /osisRef="1Chr./g;
	$line =~ s/osisRef="1 Ch /osisRef="1Chr./g;
	$line =~ s/osisRef="1Ch /osisRef="1Chr./g;
	$line =~ s/osisRef="2 Chronicles /osisRef="2Chr./g;
	$line =~ s/osisRef="2 Ch /osisRef="2Chr./g;
	$line =~ s/osisRef="2Ch /osisRef="2Chr./g;
	$line =~ s/osisRef="Ezra /osisRef="Ezra./g;
	$line =~ s/osisRef="Ezr /osisRef="Ezra./g;
	$line =~ s/osisRef="Nehemiah /osisRef="Neh./g;
	$line =~ s/osisRef="Ne /osisRef="Neh./g;
	$line =~ s/osisRef="Esther /osisRef="Esth./g;
	$line =~ s/osisRef="Es /osisRef="Esth./g;
	$line =~ s/osisRef="Job /osisRef="Job./g;
	$line =~ s/osisRef="Psalms /osisRef="Ps./g;
	$line =~ s/osisRef="Psalm /osisRef="Ps./g;
	$line =~ s/osisRef="Ps /osisRef="Ps./g;
	$line =~ s/osisRef="Proverbs /osisRef="Prov./g;
	$line =~ s/osisRef="Pr /osisRef="Prov./g;
	$line =~ s/osisRef="Ecclesiastes /osisRef="Eccl./g;
	$line =~ s/osisRef="Ec /osisRef="Eccl./g;
	$line =~ s/osisRef="Song /osisRef="Song./g;
	$line =~ s/osisRef="SS /osisRef="Song./g;
	$line =~ s/osisRef="Isaiah /osisRef="Isa./g;
	$line =~ s/osisRef="Isa /osisRef="Isa./g;
	$line =~ s/osisRef="Jeremiah /osisRef="Jer./g;
	$line =~ s/osisRef="Je /osisRef="Jer./g;
	$line =~ s/osisRef="Lamentations /osisRef="Lam./g;
	$line =~ s/osisRef="La /osisRef="Lam./g;
	$line =~ s/osisRef="Ezekiel /osisRef="Ezek./g;
	$line =~ s/osisRef="Eze /osisRef="Ezek./g;
	$line =~ s/osisRef="Daniel /osisRef="Dan./g;
	$line =~ s/osisRef="Da /osisRef="Dan./g;
	$line =~ s/osisRef="Hosea /osisRef="Hos./g;
	$line =~ s/osisRef="Ho /osisRef="Hos./g;
	$line =~ s/osisRef="Joel /osisRef="Joel./g;
	$line =~ s/osisRef="Joe /osisRef="Joel./g;
	$line =~ s/osisRef="Amos /osisRef="Amos./g;
	$line =~ s/osisRef="Am /osisRef="Amos./g;
	$line =~ s/osisRef="Obadiah /osisRef="Obad./g;
	$line =~ s/osisRef="Ob /osisRef="Obad./g;
	$line =~ s/osisRef="Jonah /osisRef="Jonah./g;
	$line =~ s/osisRef="Jon /osisRef="Jonah./g;
	$line =~ s/osisRef="Micah /osisRef="Mic./g;
	$line =~ s/osisRef="Mi /osisRef="Mic./g;
	$line =~ s/osisRef="Nahum /osisRef="Nah./g;
	$line =~ s/osisRef="Na /osisRef="Nah./g;
	$line =~ s/osisRef="Habakkuk /osisRef="Hab./g;
	$line =~ s/osisRef="Hab /osisRef="Hab./g;
	$line =~ s/osisRef="Zephaniah /osisRef="Zeph./g;
	$line =~ s/osisRef="Zep /osisRef="Zeph./g;
	$line =~ s/osisRef="Haggai /osisRef="Hag./g;
	$line =~ s/osisRef="Hag /osisRef="Hag./g;
	$line =~ s/osisRef="Zechariah /osisRef="Zech./g;
	$line =~ s/osisRef="Zec /osisRef="Zech./g;
	$line =~ s/osisRef="Malachi /osisRef="Mal./g;
	$line =~ s/osisRef="Mal /osisRef="Mal./g;
	$line =~ s/osisRef="Matthew /osisRef="Matt./g;
	$line =~ s/osisRef="Mat /osisRef="Matt./g;
	$line =~ s/osisRef="Mt /osisRef="Matt./g;
	$line =~ s/osisRef="Mark /osisRef="Mark./g;
	$line =~ s/osisRef="Mar /osisRef="Mark./g;
	$line =~ s/osisRef="Mk /osisRef="Mark./g;
	$line =~ s/osisRef="Luke /osisRef="Luke./g;
	$line =~ s/osisRef="Lu /osisRef="Luke./g;
	$line =~ s/osisRef="Lk /osisRef="Luke./g;
	$line =~ s/osisRef="John /osisRef="John./g;
	$line =~ s/osisRef="Joh /osisRef="John./g;
	$line =~ s/osisRef="Acts /osisRef="Acts./g;
	$line =~ s/osisRef="Ac /osisRef="Acts./g;
	$line =~ s/osisRef="Romans /osisRef="Rom./g;
	$line =~ s/osisRef="Ro /osisRef="Rom./g;
	$line =~ s/osisRef="Rm /osisRef="Rom./g;
	$line =~ s/osisRef="1 Corinthians /osisRef="1Cor./g;
	$line =~ s/osisRef="1 Co /osisRef="1Cor./g;
	$line =~ s/osisRef="1Co /osisRef="1Cor./g;
	$line =~ s/osisRef="2 Corinthians /osisRef="2Cor./g;
	$line =~ s/osisRef="2 Co /osisRef="2Cor./g;
	$line =~ s/osisRef="2Co /osisRef="2Cor./g;
	$line =~ s/osisRef="Galatians /osisRef="Gal./g;
	$line =~ s/osisRef="Ga /osisRef="Gal./g;
	$line =~ s/osisRef="Ephesians /osisRef="Eph./g;
	$line =~ s/osisRef="Ep /osisRef="Eph./g;
	$line =~ s/osisRef="Philippians /osisRef="Phil./g;
	$line =~ s/osisRef="Phili /osisRef="Phil./g;
	$line =~ s/osisRef="Php /osisRef="Phil./g;
	$line =~ s/osisRef="Colossians /osisRef="Col./g;
	$line =~ s/osisRef="Col /osisRef="Col./g;
	$line =~ s/osisRef="1 Thessalonians /osisRef="1Thess./g;
	$line =~ s/osisRef="1 Th /osisRef="1Thess./g;
	$line =~ s/osisRef="1Th /osisRef="1Thess./g;
	$line =~ s/osisRef="2 Thessalonians /osisRef="2Thess./g;
	$line =~ s/osisRef="2 Th /osisRef="2Thess./g;
	$line =~ s/osisRef="2Th /osisRef="2Thess./g;
	$line =~ s/osisRef="1 Timothy /osisRef="1Tim./g;
	$line =~ s/osisRef="1 Ti /osisRef="1Tim./g;
	$line =~ s/osisRef="1Ti /osisRef="1Tim./g;
	$line =~ s/osisRef="2 Timothy /osisRef="2Tim./g;
	$line =~ s/osisRef="2 Ti /osisRef="2Tim./g;
	$line =~ s/osisRef="2Ti /osisRef="2Tim./g;
	$line =~ s/osisRef="Titus /osisRef="Tit./g;
	$line =~ s/osisRef="Tit /osisRef="Tit./g;
	$line =~ s/osisRef="Philemon /osisRef="Phlm./g;
	$line =~ s/osisRef="Phile /osisRef="Phlm./g;
	$line =~ s/osisRef="Phm /osisRef="Phlm./g;
	$line =~ s/osisRef="Hebrews /osisRef="Heb./g;
	$line =~ s/osisRef="Heb /osisRef="Heb./g;
	$line =~ s/osisRef="He /osisRef="Heb./g;
	$line =~ s/osisRef="James /osisRef="Jas./g;
	$line =~ s/osisRef="Ja /osisRef="Jas./g;
	$line =~ s/osisRef="1 Peter /osisRef="1Pet./g;
	$line =~ s/osisRef="1 Pe /osisRef="1Pet./g;
	$line =~ s/osisRef="1Pe /osisRef="1Pet./g;
	$line =~ s/osisRef="2 Peter /osisRef="2Pet./g;
	$line =~ s/osisRef="2 Pe /osisRef="2Pet./g;
	$line =~ s/osisRef="2Pe /osisRef="2Pet./g;
	$line =~ s/osisRef="1 John /osisRef="1John./g;
	$line =~ s/osisRef="1 Jo /osisRef="1John./g;
	$line =~ s/osisRef="1Jo /osisRef="1John./g;
	$line =~ s/osisRef="2 John /osisRef="2John./g;
	$line =~ s/osisRef="2 Jo /osisRef="2John./g;
	$line =~ s/osisRef="2Jo /osisRef="2John./g;
	$line =~ s/osisRef="3 John /osisRef="3John./g;
	$line =~ s/osisRef="3 Jo /osisRef="3John./g;
	$line =~ s/osisRef="3Jo /osisRef="3John./g;
	$line =~ s/osisRef="Jude /osisRef="Jude./g;
	$line =~ s/osisRef="Revelation /osisRef="Rev./g;
	$line =~ s/osisRef="Re /osisRef="Rev./g;
	$line =~ s/osisRef="Gen /osisRef="Gen./g;
	$line =~ s/osisRef="Exod /osisRef="Exod./g;
	$line =~ s/osisRef="Lev /osisRef="Lev./g;
	$line =~ s/osisRef="Num /osisRef="Num./g;
	$line =~ s/osisRef="Deut /osisRef="Deut./g;
	$line =~ s/osisRef="Josh /osisRef="Josh./g;
	$line =~ s/osisRef="Judg /osisRef="Judg./g;
	$line =~ s/osisRef="Ruth /osisRef="Ruth./g;
	$line =~ s/osisRef="1Sam /osisRef="1Sam./g;
	$line =~ s/osisRef="2Sam /osisRef="2Sam./g;
	$line =~ s/osisRef="1Kgs /osisRef="1Kgs./g;
	$line =~ s/osisRef="2Kgs /osisRef="2Kgs./g;
	$line =~ s/osisRef="1Chr /osisRef="1Chr./g;
	$line =~ s/osisRef="2Chr /osisRef="2Chr./g;
	$line =~ s/osisRef="Ezra /osisRef="Ezra./g;
	$line =~ s/osisRef="Neh /osisRef="Neh./g;
	$line =~ s/osisRef="Esth /osisRef="Esth./g;
	$line =~ s/osisRef="Job /osisRef="Job./g;
	$line =~ s/osisRef="Ps /osisRef="Ps./g;
	$line =~ s/osisRef="Prov /osisRef="Prov./g;
	$line =~ s/osisRef="Eccl /osisRef="Eccl./g;
	$line =~ s/osisRef="Song /osisRef="Song./g;
	$line =~ s/osisRef="Isa /osisRef="Isa./g;
	$line =~ s/osisRef="Jer /osisRef="Jer./g;
	$line =~ s/osisRef="Lam /osisRef="Lam./g;
	$line =~ s/osisRef="Ezek /osisRef="Ezek./g;
	$line =~ s/osisRef="Dan /osisRef="Dan./g;
	$line =~ s/osisRef="Hos /osisRef="Hos./g;
	$line =~ s/osisRef="Joel /osisRef="Joel./g;
	$line =~ s/osisRef="Amos /osisRef="Amos./g;
	$line =~ s/osisRef="Obad /osisRef="Obad./g;
	$line =~ s/osisRef="Jonah /osisRef="Jonah./g;
	$line =~ s/osisRef="Mic /osisRef="Mic./g;
	$line =~ s/osisRef="Nah /osisRef="Nah./g;
	$line =~ s/osisRef="Hab /osisRef="Hab./g;
	$line =~ s/osisRef="Zeph /osisRef="Zeph./g;
	$line =~ s/osisRef="Hag /osisRef="Hag./g;
	$line =~ s/osisRef="Zech /osisRef="Zech./g;
	$line =~ s/osisRef="Mal /osisRef="Mal./g;
	$line =~ s/osisRef="Matt /osisRef="Matt./g;
	$line =~ s/osisRef="Mark /osisRef="Mark./g;
	$line =~ s/osisRef="Luke /osisRef="Luke./g;
	$line =~ s/osisRef="John /osisRef="John./g;
	$line =~ s/osisRef="Acts /osisRef="Acts./g;
	$line =~ s/osisRef="Rom /osisRef="Rom./g;
	$line =~ s/osisRef="1Cor /osisRef="1Cor./g;
	$line =~ s/osisRef="2Cor /osisRef="2Cor./g;
	$line =~ s/osisRef="Gal /osisRef="Gal./g;
	$line =~ s/osisRef="Eph /osisRef="Eph./g;
	$line =~ s/osisRef="Phil /osisRef="Phil./g;
	$line =~ s/osisRef="Col /osisRef="Col./g;
	$line =~ s/osisRef="1Thess /osisRef="1Thess./g;
	$line =~ s/osisRef="2Thess /osisRef="2Thess./g;
	$line =~ s/osisRef="1Tim /osisRef="1Tim./g;
	$line =~ s/osisRef="2Tim /osisRef="2Tim./g;
	$line =~ s/osisRef="Titus /osisRef="Titus./g;
	$line =~ s/osisRef="Phlm /osisRef="Phlm./g;
	$line =~ s/osisRef="Heb /osisRef="Heb./g;
	$line =~ s/osisRef="Jas /osisRef="Jas./g;
	$line =~ s/osisRef="1Pet /osisRef="1Pet./g;
	$line =~ s/osisRef="2Pet /osisRef="2Pet./g;
	$line =~ s/osisRef="1John /osisRef="1John./g;
	$line =~ s/osisRef="2John /osisRef="2John./g;
	$line =~ s/osisRef="3John /osisRef="3John./g;
	$line =~ s/osisRef="Jude /osisRef="Jude./g;
	$line =~ s/osisRef="Rev /osisRef="Rev./g;
# Reformats the osisRef attribute to comply with OSIS schema.
	# Gen.1:1-2:1
	$line =~ s/osisRef="(\d{0,1}[a-zA-Z]{2,5}).(\d{1,3}):(\d{1,3})-(\d{1,3}):(\d{1,3})/osisRef="$1.$2.$3-$1.$4.$5/g;
	# Gen.1:1-2
	$line =~ s/osisRef="(\d{0,1}[a-zA-Z]{2,5}).(\d{1,3}):(\d{1,3})-(\d{1,3})/osisRef="$1.$2-$1.$2.$4/g;
	# Gen.1-2
	$line =~ s/osisRef="(\d{0,1}[a-zA-Z]{2,5}).(\d{1,3})-(\d{1,3})/osisRef="$1.$2-$1.$3/g;
	# Gen.1:1
	$line =~ s/osisRef="(\d{0,1}[a-zA-Z]{2,5}).(\d{1,3}):(\d{1,3})/osisRef="$1.$2.$3/g;
	# Gen.1
	$line =~ s/osisRef="(\d{0,1}[a-zA-Z]{2,5}).(\d{1,3})/osisRef="$1.$2/g;

	print OUTF "$line";
}

close (OUTF);
